// ================================================== //
//       Contact Center Monitor interface module      //
//                     Version 2.2.6                  //
//                                                    //
// ---------- (c) AO SBK 2015-2022 ---------- //
// ================================================== //

var monitorData = {
  group: {},
  agent: {},
  call: {},
  campaign: {},
};
var lineClasses = {
  busy: 's1',
  free: 'free',
  ready: 's3',
  ringing: 's3',
  waiting: 's2',
};

function resetMonitorData() {
  monitorData = {
    group: {},
    agent: {},
    call: {},
    campaign: {},
  };
}

function getDirectoryRecord(name) {
  for(var i = 0; i < directoryData.length; i++) {
    if(directoryData[i].name == name) {
      return directoryData[i];
    }
  }
  return null;
}

function resetMonitor() {
  resetMonitorData();
  $('#monContainer').html('');
  // <cg-agents>
  $('#cga').html('');
  // </cg-agents>
  refreshMonitor();
}

function updateMonitor(objects) {
  var objectsNames = Object.keys(objects);
  var iKey;
  for(var i = 0; i < objectsNames.length; ++i) {
    iKey = objectsNames[i];
    if(objects[iKey].type == 'group') {
      monGroup(objects[iKey]);
    } else if(objects[iKey].type == 'agent') {
      monAgent(objects[iKey]);
    } else if(objects[iKey].type == 'call') {
      monCall(objects[iKey]);
    } else if(objects[iKey].type == 'campaign') {
      monCamp(objects[iKey]);
    }
  }
}

function monGroup(o) {
  if(o.op == 'add') {
    monitorData.group[o.name] = o.data;
    monitorData.group[o.name].operators = 0;
    monitorData.group[o.name].busy = 0;
    monitorData.group[o.name].available = 0;
    monitorData.group[o.name].unavailable = 0;
    monitorData.group[o.name].queueLength = 0;
    monitorData.group[o.name].missed = 0;
    var camp = newCampHtml();
    if(!sessionParams.isDomainAdmin) {
      camp = '';
    }
    var on = '<a title="' + o.name + '">' + (o.data.realName && o.data.realName.length > 0 ? (o.data.realName.length > 30 ? o.data.realName.substring(0,30) + '...' : o.data.realName) : o.name) + '</a>';
    var html = '<li id="mon_' + o.name + '" class="bg">' +
      '<ul class="head bg">' +
      '<li class="td0"><span class="number">' + o.data.extensions.join(',') + '</span>' +
      on + '</li>' +
      '<li class="td td2">' + monitorData.group[o.name].operators + '</li>' +
      '<li class="td td3">' + monitorData.group[o.name].busy + '</li>' +
      '<li class="td td4 color">' + monitorData.group[o.name].available + '</li>' +
      '<li class="td td7">0</li>' +
      '<li class="td td5">' + monitorData.group[o.name].unavailable + '</li>' +
      '<li class="td td6">' + monitorData.group[o.name].queueLength + '</li>' +
      //'<li class="td td7 color">' + monitorData.group[o.name].missed + '</li>' +
      '<li class="camp"><a>' + translate('off') + '</a></li>' +
      '</ul>' +
      '<p class="monAgentFilterContainer"><span class="monAgentFilter select"><select class="multipleSelect" multiple></select></span></p>' +
      '<ul class="items"></ul>' +
      '<div class="qCamp detals clr">' + camp + '</div>' +
      '<p class="turn woqt"><b class="pointer border">' + translate('inqueue') + '</b> <span></span></p>' +
      '<span class="close"></span>' +
      '</li>';
    $('#monContainer').append(html);
    // campaign
    if(!sessionParams.isDomainAdmin) {
      $('.camp').hide();
    } else {
      var cmpn = $('#mon_' + o.name + ' .qCamp');
      setNewCampHandlers(cmpn);
    }
    // <agents>
    var agents = o.data.agents;
    if(sessionParams && sessionParams.agentSorting === 'byName') agents.sort(function(a, b) { return a.localeCompare(b, undefined, {numeric: true}); });
    var iKey;
    for(var i = 0; i < agents.length; ++i) {
      iKey = agents[i];
      if(monitorData.agent[iKey]) {
        monitorData.agent[iKey].groups.push(o.name);
        var prn = iKey;
        if(prn.length > 11) {
          prn = '<span title="' + iKey + '">' + iKey.substring(0,11) + '...</span>';
        }
        $('#mon_' + o.name + ' .items').append(
          '<li class="' + monitorData.agent[iKey].status +
          '" data-value="' + iKey + '">' +
          '<div class="info bg">' +
          '<p class="name">' + prn + '</p>' +
          '<i class="description center">' + getTruncName(monitorData.agent[iKey].realName, 30) + '</i>' +
          '<div class="detals">' +
          make_select('avunav', 'agentStatus', (monitorData.agent[iKey].statusInfo ?
              monitorData.agent[iKey].statusInfo : monitorData.agent[iKey].status)) +
          '<p class="tStatusModified"></p>' +
          '<p class="lines"></p>' +
          '<p class="i2"></p>' +
          '</div>' +
          '</div>' +
          '<span class="listener button1 bg" data-value="eavesdrop" title="' + translate('listen') + '"></span>' +
          '<span class="listener button2 bg" data-value="whisper" title="' + translate('whisper') + '"></span>' +
          '<span class="listener button3 bg" data-value="conference" title="' + translate('conference') + '"></span>' +
          '</li>'
        );
        setTAgentStatusModified(
          $('#mon_' + o.name + ' .items li[data-value="' + iKey + '"]'),
          monitorData.agent[iKey].tStatusModified
        );
        $('#mon_' + o.name + ' .items li[data-value="' + iKey + '"] .agentStatus').on('change', function() {
          setAgentStatus($(this).parents('li').attr('data-value'), $(this).val(), $(this).attr('data-value'));
        });
        // <listeners>
        $('#mon_' + o.name + ' .items li[data-value="' + iKey + '"] .listener').on('click', function() {
          var agentName = $(this).parents('li').attr('data-value');
          var mode = $(this).hasClass('active') ? null : $(this).attr('data-value');
          var firstLineCall = $(this).parent().find('.lines > span:first-child').attr('data-value');
          if(firstLineCall) {
            setCallListener(firstLineCall, mode);
          }
        });
        // </listeners>
        monitorData.group[o.name].operators++;
        if(monitorData.agent[iKey].status == 'on') {
          monitorData.group[o.name].available++;
        } else {
          monitorData.group[o.name].unavailable++;
        }
      }
    } // </agents>
    if(o.data.queue) {
      monitorData.group[o.name].queueLength = o.data.queue.length;
    }
    updateMonGroupHead(o.name);
    setMonGroupHandlers($('#mon_' + o.name));
  } else if(o.op == 'update') {
    $.extend(monitorData.group[o.name], o.data);
    flushNulls('group', o.name);
    if(o.data.queue) {
      monitorData.group[o.name].queueLength = o.data.queue.length;
    }
    updateMonGroupHead(o.name);
  } else if(o.op == 'remove') {
    delete monitorData.group[o.name];
    $('#mon_' + o.name).remove();
  } else {
    //
  }
}

function updateAgentPanel(data) {
  var c = $('#agentPanel');
  c.find('.select .value').attr('data-value', data.statusInfo ? data.status + ':' + data.statusInfo : data.status);
  c.find('.select .value').text(data.statusInfo ? translate(['agentStatusInfo', data.status, data.statusInfo]) : translate(['agentStatus', data.status]));
}

function monAgent(o) {
  if(o.op == 'add') {
    monitorData.agent[o.name] = o.data;
    monitorData.agent[o.name].groups = [];
    // <agentPanel>
    if(o.name == sessionParams.name) {
      updateAgentPanel(o.data);
    } else {
      // <cg-agents>
      addCGAgent(o.name);
      // </cg-agents>
    }
    // </agentPanel>
  } else if(o.op == 'update') {
    // <agentPanel>
    if(o.name == sessionParams.name) {
      updateAgentPanel(o.data);
    }
    // </agentPanel>
    // previous state
    var prevStatus = monitorData.agent[o.name].status;
    var prevStatusInfo = monitorData.agent[o.name].statusInfo;
    // update state
    $.extend(monitorData.agent[o.name], o.data);
    flushNulls('agent', o.name);
    var agent = monitorData.agent[o.name];
    var sel = $('#monContainer .items li[data-value="' + o.name + '"]');
    var className;
    var s;
    if(agent.status != prevStatus || agent.statusInfo != prevStatusInfo) {
      displayAgentStatus(o.name, agent.status);
      className = agent.status;
      if(agent.statusInfo) {
        var si = translate(['agentStatusInfo', agent.status]);
        if(si[agent.statusInfo]) s = si[agent.statusInfo];
        else s = translate(['agentStatus', agent.status]);
      } else {
        s = translate(['agentStatus', agent.status]);
      }
      if(agent.status == 'on' && prevStatus != 'on') {
        for(var i = 0; i < agent.groups.length; ++i) {
          monitorData.group[agent.groups[i]].available++;
          monitorData.group[agent.groups[i]].unavailable--;
          updateMonGroupHead(agent.groups[i]);
        }
      } else if(prevStatus == 'on' && agent.status != 'on') {
        for(var i = 0; i < agent.groups.length; ++i) {
          monitorData.group[agent.groups[i]].unavailable++;
          monitorData.group[agent.groups[i]].available--;
          updateMonGroupHead(agent.groups[i]);
        }
      }
      sel.removeClass().addClass(className);
      sel.find('.select .value').text(s);
      setTAgentStatusModified(sel, agent.tStatusModified);
      // <cg-agents>
      $('#cga div[data-value="' + o.name + '"] .status').text(s);
      $('#cga div[data-value="' + o.name + '"] .icon').removeClass().addClass('icon').addClass(className);
      // </cg-agents>
    }
  } else if(o.op == 'remove') {
    delete monitorData.agent[o.name];
    $('#monContainer .items li[data-value="' + o.name + '"]').remove();
    // <cg-agents>
    $('#cga div[data-value="' + o.name + '"]').remove();
    // </cg-agents>
  } else {
    //
  }
}

function setTAgentStatusModified(sel, tStatusModified) {
  if(!sel.length || !tStatusModified) return;
  var cnt = sel.find('.tStatusModified');
  var oldTimerId = cnt.attr('data-value');
  if(oldTimerId) clearInterval(parseInt(oldTimerId));
  var ts = new Date(tStatusModified);
  /*
  var mm = ts.getMinutes();
  mm = mm < 10 ? '0' + mm : mm;
  var hh = ts.getHours();
  hh = hh < 10 ? '0' + hh : hh;
  var dd = ts.getDate();
  dd = dd < 10 ? '0' + dd : dd;
  var mon = ts.getMonth() + 1;
  mon = mon < 10 ? '0' + mon : mon;
  sel.find('.tStatusModified').text(dd+'-'+mon+' '+hh+':'+mm);
  */
  var timerId = setInterval(function() {
    if(!sel.length || cnt.attr('data-value') != timerId) {
      clearInterval(timerId);
      return;
    }
    now = new Date();
    ms = now.getTime() - ts;
    tts = +(ms/1000).toFixed();
    ss = tts % 60;
    mm = Math.floor(tts/60);
    hh = Math.floor(tts/3600);
    ss = ss > 59 ? ss - 60 : ss;
    mm = mm > 59 ? mm % 60 : mm;
    hh = hh < 10 ? '0' + hh : hh;
    mm = mm < 10 ? '0' + mm : mm;
    ss = ss < 10 ? '0' + ss : ss;
    //sel.text(hh + ':' + mm + ':' + ss);
    cnt.text(hh + ':' + mm + ':' + ss);
  }, 1000);
  cnt.attr('data-value', timerId);
  if(!sel.find('.lines').children().length) sel.find('.tStatusModified').show();
}

function displayLineCall(agentName, callID) {
  if(monitorData.call[callID]) {
    var lc = $(
      '#monContainer .items li[data-value="' + agentName + '"] span[data-value="' + callID + '"]'
    );
    var info;
    if(monitorData.call[callID].callerIsAgent) {
      if(monitorData.call[callID].callFrom == agentName) {
        // current agent initiates call
        info = monitorData.call[callID].callTo;
      } else {
        // another agent initiates call
        info = monitorData.call[callID].callFrom;
      }
    } else {
      info = monitorData.call[callID].callFrom;
    }
    lc.find('a').text(getUserPart(info));
    if(monitorData.call[callID].state == 'connected' && monitorData.call[callID].tConnected) {
      busyTimer(lc.find('.busyTime'), monitorData.call[callID].tConnected);
      lc.find('a').addClass('connected');
    }
    lc.removeClass('blank');
    // <agentPanel>
    if(agentName == sessionParams.name) {
      var sel = $('#selfLines a[data-value="' + callID + '"]');
      if(!sel.length) {
        sel.append('<a data-value="' + callID + '">' + getUserPart(info) + '</a>');
      } else {
        sel.text(getUserPart(info));
        sel.show();
      }
      $('#selfQueue a[data-value="' + callID + '"]').remove();
    }
    // </agentPanel>
  }
}

function displayAgentStatus(agentName, status) {
  var s;
  if(status == 'on') {
    s = translate('available');
  } else if(status == 'off') {
    s = translate('unavailable');
  }
  var statusInfo = monitorData.agent[agentName].statusInfo;
  var sel = $('#monContainer .items li[data-value="' + agentName + '"]');
  sel.find('select').val(status).siblings('.value').text(s);
  // <cg-agents>
  $('#cga div[data-value="' + agentName + '"] .status').text(s);
  $('#cga div[data-value="' + agentName + '"] .icon').removeClass().addClass('icon').addClass(status);
  // </cg-agents>
  sel.removeClass().addClass(status);
  // <monAgentFilter>
  $('#monContainer .items li[data-value="' + agentName + '"]').each(function() {
    var mafs = $(this).parent().parent().find('.monAgentFilter select');
    if(mafs.val() && mafs.val().length && mafs.val()[0] != '') {
      if((mafs.val().indexOf(status) >= 0 && statusInfo == null) || mafs.val().indexOf(statusInfo) >= 0) {
        $(this).show();
      } else {
        $(this).hide();
      }
    }
  });
  var cafs = $('#str_sc_ag select');
  if(cafs.val()) {
    if((cafs.val() == status && statusInfo == null) || cafs.val() == statusInfo) {
      $('#cga div[data-value="' + agentName + '"]').show();
    } else {
      $('#cga div[data-value="' + agentName + '"]').hide();
    }
  }
  // </monAgentFilter>
}

function monCall(o) {
  //$('#callMon_totalCalls').text(getKeysCount(monitorData.call));
  if(o.op == 'add') {
    updateCallMon(o.name, o.data);
    monitorData.call[o.name] = o.data;
    if(o.data.peers) {
      displayCall(o.name);
    }
    if(o.data.campaign && o.data.client && monitorData.campaign[o.data.campaign]) {
      displayCampCall(o.name);
    }
    // <cg-card>
    if(o.data.peers) {
      var peers = Object.keys(o.data.peers);
      var iKey;
      for(var i = 0; i < peers.length; ++i) {
        iKey = peers[i];
        if(o.data.peers[iKey].name == sessionParams.name && o.data.peers[iKey].type != 'listener') {
          displayCGCard(o.data.CGCard, $('#cgc_form'));
          delete monitorData.call[o.name].CGCard;
          break;
        }
      }
    }
    // </cg-card>
    document.getElementById('callMon_totalCalls').textContent = getKeysCount(monitorData.call);
    return;
  } else if(o.op == 'update') {
    updateCallMon(o.name, o.data);
    monitorData.call[o.name].oldPeers = monitorData.call[o.name].peers;
    $.extend(monitorData.call[o.name], o.data);
    flushNulls('call', o.name);
    if(o.data.notes && $('#callNotes').length && $('#callNotes').attr('data-value') == o.name) displayCallNoteForm(o.name, getCallQueue(monitorData.call[o.name]));
    if(o.data.peers) {
      displayCall(o.name);
      // <crmCard>
      var peers = Object.keys(o.data.peers);
      var iKey;
      for(var i = 0; i < peers.length; ++i) {
        iKey = peers[i];
        if(o.data.peers[iKey].name == sessionParams.name && o.data.peers[iKey].type != 'listener' && o.data.peers[iKey].dir != 'out' && card != o.name) {
          if(sessionParams.browserNotifications == 'on') {
            showNotify(o.name);
            displayCrmCard(o.name);
            card = o.name;
          }
          break;
        }
      }
      // </crmCard>
    }
    // <cg-card>
    if(monitorData.call[o.name].CGCard) {
      var peers = Object.keys(monitorData.call[o.name].peers);
      var iKey;
      for(var i = 0; i < peers.length; ++i) {
        iKey = peers[i];
        if(monitorData.call[o.name].peers[iKey].name == sessionParams.name && monitorData.call[o.name].peers[iKey].type != 'listener') {
          displayCGCard(monitorData.call[o.name].CGCard, $('#cgc_form'));
          delete monitorData.call[o.name].CGCard;
          break;
        }
      }
    }
    // </cg-card>
    document.getElementById('callMon_totalCalls').textContent = getKeysCount(monitorData.call);
    return;
  } else if(o.op == 'remove') {
    delCallMon(o.name);
    deletePeers(o.name);
    deleteCallFromSausages(o.name);
    if(mute == o.name) mute = null;
    if(onservice == o.name) onservice = null;
    if(refer == o.name) refer = null;
    if(card == o.name) card = null;
    updateActionButtons();
    delete monitorData.call[o.name];
    document.getElementById('callMon_totalCalls').textContent = getKeysCount(monitorData.call);
    return;
  } else {
    // unknown op
  }
}

function monCamp(o) {
  if(o.op == 'add') {
    monitorData.campaign[o.name] = o.data;
    if(monitorData.group[o.data.qName]) {
      assignCamp(o.name, o.data.qName);
      displayCampStatus(o.name, o.data.status);
      initCampHtml(o.name);
      updateCampControls(o.name);
      fixQCampWidth(o.name);
    }
    return;
  } else if(o.op == 'update') {
    $.extend(monitorData.campaign[o.name], o.data);
    displayCampStatus(o.name, o.data.status);
    updateCampControls(o.name);
    fixQCampWidth(o.name);
    return;
  } else if(o.op == 'remove') {
    delete monitorData.campaign[o.name];
    unsetCamp(o.name);
    return;
  } else {
    // unknown op
  }
}

function getCrmId(callID) {
  var id = null;
  if(monitorData.call[callID].crm) {
    if(monitorData.call[callID].crm[0]) {
      id = monitorData.call[callID].crm[0].id;
    }
  }
  return id;
}

function getCrmName(callID) {
  var name = '';
  var callObj = monitorData.call[callID];
  if(callObj.fromName) {
    name = callObj.fromName;
  }
  if(callObj.crm) {
    if(callObj.crm[0] && callObj.crm[0].name) {
      name = callObj.crm[0].name;
    }
  } else if(callObj.CGCard && callObj.CGCard.cgcard) {
    if(callObj.CGCard.cgcard.fn) name = callObj.CGCard.cgcard.fn;
    if(callObj.CGCard.cgcard.ln) name = (name ? name + ' ' : '') + callObj.CGCard.cgcard.ln;
  }
  return name;
}

function getCrmEmail(callID) {
  var email = '';
  var callObj = monitorData.call[callID];
  if(isNaN(getUserPart(callObj.callFrom)) && callObj.callFrom.indexOf('@') >= 0) {
    email = callObj.callFrom;
  }
  if(callObj.crm) {
    if(callObj.crm[0] && callObj.crm[0].email) {
      email = callObj.crm[0].email;
    }
  } else if(callObj.CGCard && callObj.CGCard.cgcard) {
    if(callObj.CGCard.cgcard.email && callObj.CGCard.cgcard.email.length) {
      email = callObj.CGCard.cgcard.email[0].email;
    }
  }
  return email;
}

function getCrmUrl(callID) {
  var url = '';
  var callObj = monitorData.call[callID];
  if(callObj.crm) {
    if(callObj.crm[0]) {
      url = callObj.crm[0].url;
    }
  }
  return url;
}

function deletePeers(callID) {
  var noear;
  var peers = Object.keys(monitorData.call[callID].peers);
  var iKey;
  for(var i = 0; i < peers.length; ++i) {
    iKey = peers[i];
    if(monitorData.call[callID].peers[iKey].type == 'listener' && monitorData.call[callID].peers[iKey].name == sessionParams.name) {
      noear = monitorData.call[callID].peers[iKey].mode;
      break;
    }
  }
  for(var i = 0; i < peers.length; ++i) {
    iKey = peers[i];
    if(monitorData.call[callID].peers[iKey].isAgent && noear) {
      var ac = $('#monContainer .items li[data-value="' + monitorData.call[callID].peers[iKey].name + '"]');
      ac.children('.listener[data-value="' + noear + '"]').removeClass('active');
    }
    if(monitorData.call[callID].peers[iKey].state == 'inqueue' && monitorData.group[monitorData.call[callID].peers[iKey].qName]) {
      monitorData.group[monitorData.call[callID].peers[iKey].qName].missed++;
      updateMonGroupHead(monitorData.call[callID].peers[iKey].qName);
    }
    $('a[data-value="' + iKey + '"]').remove();
  }
}

function deleteCallFromSausages(callID) {
  $('#monContainer span[data-value="' + callID + '"]').each(function() {
    var lines = $(this).parent();
    var detals = lines.parent();
    $(this).remove();
    if(!lines.children().length) {
      var a = detals.parent().parent().attr('data-value');
      if(a) {
        var peer;
        var peers = Object.keys(monitorData.call[callID].peers);
        var iKey;
        for(var i = 0; i < peers.length; ++i) {
          iKey = peers[i];
          if(monitorData.call[callID].peers[iKey].name == a) {
            peer = monitorData.call[callID].peers[iKey];
            break;
          }
        }
        if(peer && peer.state != 'connecting' && peer.state != 'provisioned' && peer.type != 'listener') {
          var q = detals.parent().parent().parent().parent().attr('id').substring(4);
          if(q) {
            monitorData.group[q].busy--;
            updateMonGroupHead(q);
          }
        }
      }
      //detals.children('.select').show();
      detals.children('.tStatusModified').show();
    }
  });
  // <agentPanel>
  $('#selfLines span[data-value="' + callID + '"]').remove();
  $('#selfQueue span[data-value="' + callID + '"]').remove();
  // </agentPanel>
  // <cg-agents>
  $('#cga span[data-value="' + callID + '"]').each(function() {
    var cgac = $(this).parents('.cg_agent');
    var an = cgac.attr('data-value');
    cgac.find('.status[data-value="' + callID + '"]').text(
        monitorData.agent[an].statusInfo ? translate(['agentStatusInfo', monitorData.agent[an].status, monitorData.agent[an].statusInfo]) : translate(['agentStatus', monitorData.agent[an].status]));
    cgac.find('.status[data-value="' + callID + '"]').attr('data-value', '');
    cgac.find('.icon').removeClass().addClass('icon').addClass(monitorData.agent[an].status);
    cgac.find('.busyTime[data-value="' + callID + '"]').text('');
    cgac.find('.busyTime[data-value="' + callID + '"]').attr('data-value', '');
    if(!$(this).hasClass('status') && !$(this).hasClass('busyTime')) {
      $(this).remove();
    }
  });
  // </cg-agents>
  // campaign
  removeCampCall(callID);
}

function flushNulls(type, name) {
  var keyArr = Object.keys(monitorData[type][name]);
  var iKey;
  for(var i = 0; i < keyArr.length; ++i) {
    iKey = keyArr[i];
    if(monitorData[type][name][iKey] == '#NULL#') delete monitorData[type][name][iKey];
  }
}

function setMonGroupHandlers(sel) {
  sel.find('.turn b').on('click', function() {
    $(this).parent().toggleClass('woqt');
  });
  sel.children('.head').on('click',function() {
    sel.find('.qCamp').hide();
    sel.find('.items').show();
    sel.find('.turn').show();
		var t = $(this);
		var li = t.parent();
    if(t.children('.camp').hasClass('active')) {
      t.children('.camp').removeClass('active');
    }
		if(li.hasClass('active')){
			t.siblings('.close').stop().fadeOut();
			li.removeClass('active');
			li.stop().animate({'height':t.outerHeight()},500);
		}
		else {
			var h = t.outerHeight() + t.siblings('.items').outerHeight() + t.siblings('.monAgentFilterContainer').outerHeight() + t.siblings('.turn').outerHeight();
      if(h < 200) h = 210;
			li.addClass('active');
			li.stop().animate({'height':h},700,function(){
				t.stop().siblings('.close').fadeIn();
			});
		}
		return false;
	});
  sel.find('.head .camp').on('click',function(e) {
    e.stopPropagation();
    sel.find('.items').hide();
    sel.find('.turn').hide();
    sel.find('.qCamp').show();
    sel.find('.qCamp .turn').show();
		var t = $(this);
		var head = t.parent();
    var li = head.parent();
    if(li.hasClass('active')) {
      li.removeClass('active');
    }
		if(t.hasClass('active')){
			head.siblings('.close').stop().fadeOut();
			t.removeClass('active');
			li.stop().animate({'height':t.outerHeight()},500);
      sel.find('.qCamp .campListContainer').html('');
      sel.find('.qCamp li.bg2').removeClass('bg2');
		} else {
			var h = head.outerHeight() + head.siblings('.qCamp').outerHeight();
			t.addClass('active');
			li.stop().animate({'height':h},700,function(){
				head.stop().siblings('.close').fadeIn();
			});
		}
		return false;
	});
  sel.children('.close').on('click',function() {
		if($(this).parent().hasClass('active')) {
      $(this).siblings('.head').trigger('click');
    } else {
      $(this).siblings('.head').children('.camp').trigger('click');
    }
	});
  // <monAgentFilter>
  var mafs = sel.find('.monAgentFilter select');
  mafs.append('<option value="" selected="selected">' + translate('allagents') + '</option>');
  var statusInfo = translate('agentStatusInfo');
  if(typeof(statusInfo) != 'object') statusInfo = {};
  var statusKeyArr = Object.keys(translate('agentStatus'));
  var iKey;
  for(var i = 0; i < statusKeyArr.length; ++i) {
    iKey = statusKeyArr[i];
    if(iKey == 'acw') continue;
    if(statusInfo[iKey]) {
      var infoKeyArr = Object.keys(statusInfo[iKey]);
      var jKey;
      for(var j = 0; j < infoKeyArr.length; ++j) {
        jKey = infoKeyArr[j];
        //mafs.append('<option value="' + jKey + '" data-value="' + iKey + '">' + statusInfo[iKey][jKey] + '</option>');
        mafs.append('<option value="' + jKey + '">' + statusInfo[iKey][jKey] + '</option>');
      }
    }
    mafs.append('<option value="' + iKey + '">' + translate(['agentStatus', iKey]) + '</option>');
  }
  mafs.on('change', function() {
    var qName = sel.attr('id').substr(4);
    var agents = Object.keys(monitorData.group[qName].agents);
    if($(this).val() && $(this).val().length && $(this).val()[0] != '') {
      for(var i = 0; i < agents.length; ++i) {
        iKey = agents[i]; var iAgent = monitorData.agent[monitorData.group[qName].agents[iKey]];
        if(($(this).val().indexOf(iAgent.status) >= 0 && iAgent.statusInfo == null) || $(this).val().indexOf(iAgent.statusInfo) >= 0) {
          sel.find('li[data-value="' + monitorData.group[qName].agents[iKey] + '"]').show();
        } else {
          sel.find('li[data-value="' + monitorData.group[qName].agents[iKey] + '"]').hide();
        }
      }
    } else {
      for(var i = 0; i < agents.length; ++i) {
        iKey = agents[i];
        sel.find('li[data-value="' + monitorData.group[qName].agents[iKey] + '"]').show();
      }
    }
    adjustGroupHeight(sel);
  });
  // </monAgentFilter>
  sel.find('.select').each(function() {
    if($(this).parents('.qCamp').length) {
      return;
    }
    var ignore = false;
    if($(this).children('.agentStatus').length) {
      ignore = true;
    }
    if($(this).hasClass('monAgentFilter')) mul_select_html($(this));
    else select_html($(this), ignore);
  });
}

function getFirstLineCall(agentName) {
  var lines = Object.keys(monitorData.agent[agentName].lines);
  if(lines.length) return lines[0];
  return null;
}

function updateMonGroupHead(name) {
  if(monitorData.group[name].operators < 0) monitorData.group[name].operators = 0;
  if(monitorData.group[name].busy < 0) monitorData.group[name].busy = 0;
  if(monitorData.group[name].available < 0) monitorData.group[name].available = 0;
  if(monitorData.group[name].unavailable < 0) monitorData.group[name].unavailable = 0;
  var sel = $('#mon_' + name + ' .head');
  sel.find('.td0 .number').text(monitorData.group[name].extensions.join(','));
  sel.find('.td2').text(monitorData.group[name].operators);
  sel.find('.td3').text(monitorData.group[name].busy);
  if(monitorData.group[name].busy) {
    sel.find('.td3').addClass('color-red');
  } else {
    sel.find('.td3').removeClass('color-red');
  }
  sel.find('.td4').text(monitorData.group[name].available);
  if(monitorData.group[name].available) {
    sel.find('.td4').addClass('color');
  } else {
    sel.find('.td4').removeClass('color');
  }
  sel.find('.td5').text(monitorData.group[name].unavailable);
  sel.find('.td6').text(monitorData.group[name].queueLength);
  if(monitorData.group[name].queueLength) {
    sel.find('.td6').addClass('color');
  } else {
    sel.find('.td6').removeClass('color');
  }
  /*
  sel.find('.td7').text(monitorData.group[name].missed);
  if(monitorData.group[name].missed) {
    sel.find('.td7').addClass('color');
  } else {
    sel.find('.td7').removeClass('color');
  }
  */
  var agentList = monitorData.group[name].agents;
  var readyNum = monitorData.group[name].available;
  for(var i = 0; i < agentList.length; ++i) {
    if(monitorData.agent[agentList[i]].status != "on") continue;
    if(sel.parent().find('li[data-value="' + agentList[i] + '"] .lines > span').length) --readyNum;
  }
  sel.find('.td7').text(readyNum);
  if(readyNum) sel.find('.td7').addClass('color');
  else sel.find('.td7').removeClass('color');
  adjustGroupHeight(sel.parent());
}

function busyTimer(sel, tConnected) {
  var conn = Date.parse(tConnected);
  var corr = new Date();
  if(conn > corr) conn = corr;
  var now;
  var ts;
  var ms;
  var ss;
  var mm;
  var hh;
  var timerId = setInterval(function() {
    if(!sel.length || (sel.parents('.cg_agent').length && !sel.attr('data-value'))) {
      sel.text('');
      clearInterval(timerId);
      return;
    }
    now = new Date();
    ms = now.getTime() - conn;
    ts = +(ms/1000).toFixed();
    ss = ts % 60;
    mm = Math.floor(ts/60);
    hh = Math.floor(ts/3600);
    ss = ss > 59 ? ss - 60 : ss;
    mm = mm > 59 ? mm % 60 : mm;
    hh = hh < 10 ? '0' + hh : hh;
    mm = mm < 10 ? '0' + mm : mm;
    ss = ss < 10 ? '0' + ss : ss;
    //sel.text(hh + ':' + mm + ':' + ss);
    sel.text(mm + ':' + ss);
  }, 1000);
}
function handleFileUpload(files, name) {
  for (var i = 0; i < files.length; i++) {
    var fd = new FormData();
    fd.append('fileData', files[i]);
    sendFileToServer(fd, name);
    // get first file
    break;
  }
}
function sendFileToServer(formData, fileName) {
  var dt = new Date();
  var uploadID = dt.getTime();
  $.ajax({
    url: location.origin + '/Session/' + sessionId + '/UPLOAD/' + uploadID,
    type: 'POST',
    contentType:false,
    processData: false,
    data: formData,
    success: function(data) {
      if(/OK/.test(data)) {
        if(myController != null) myController.storeFile(uploadID, fileName);
      }
    },
  });
}
function displaySMSForm(callID) {
  if(!$('#sms_form').length) {
    $('body').append('<div id="sms_form"><a class="closeIcon right pointer" title="Close"></a>' +
      '<p><label>' + translate('number') + '</label>&nbsp;&nbsp;&nbsp;' +
      '<input id="smsNum" size="12" class="input bg" type="text" /><input style="display:none" id="smsList" type="file" />' +
      '<img class="pointer smsList" src="' + filesRef + 'list.svg" title="' + translate('uploadnumlist') + '" /></p>' +
      '<p style="text-align: center; margin-top: 8px;">' + translate('smstemplates') + '</p>' +
      '<div class="tList"><div class="scrollList"><div></div></div></div>' +
      '<table width="100%" cellspacing="0" cellpadding="0"><tr>' +
      '<td><textarea class="input bg" id="smsText"></textarea></td><td align="center" valign="bottom">' +
      '<button id="send_sms" class="button ff0">' + translate('send') + '</button></td></tr></table></div>'
    );
  }
  var c = $('#sms_form');
  var num;
  if(callID && monitorData.call[callID] && monitorData.call[callID].peers) {
    var peers = Object.keys(monitorData.call[callID].peers);
    var iKey;
    for(var i = 0; i < peers.length; ++i) {
      iKey = peers[i];
      if(monitorData.call[callID].peers[iKey].name != sessionParams.name) {
        num = getUserPart(monitorData.call[callID].peers[iKey].name);
        break;
      }
    }
  }
  if(num) {
    c.find('#smsNum').val(num);
  }
  c.find('.smsList').on('click', function() {
    $('#smsList').trigger('click');
  });
  $('#smsList').on('change', function() {
    if($(this)[0].files && $(this)[0].files[0]) {
      $('#smsNum').val($(this)[0].files[0].name);
    }
  });
  c.find('.closeIcon').on('click', function() {
    $(this).parent().remove();
    $('#amsg').removeClass('ff0');
  });
  c.find('#send_sms').on('click', function() {
    if($('#smsList')[0].files.length) {
      handleFileUpload($('#smsList')[0].files, '/private/settings/campaign');
      $(document).one('fileStored', function() {
        sendSMS(sessionParams.name, null, $('#smsText').val(), '/private/settings/campaign');
        c.find('.closeIcon').trigger('click');
      });
      return;
    }
    var num = $('#smsNum').val();
    if(!num) {
      return;
    }
    sendSMS(sessionParams.name, num, $('#smsText').val());
    c.find('.closeIcon').trigger('click');
  });
  getSMSTemplates();
}
function displayCrmCard(o, transf) {
  var peerID = o;
  var callID = getCallByPeer(peerID);
  if(!callID) {
    callID = peerID;
  }
  if(!monitorData.call[callID] && transf != 'conf') {
    return;
  }
  if(!$('#card').length) {
    $('body').append('<div id="card"></div>');
  }
  var CID, id, name, email, url;
  if(callID) {
    CID = getUserPart(monitorData.call[callID].callFrom);
    id = getCrmId(callID);
    name = getCrmName(callID);
    email = getCrmEmail(callID);
    url = getCrmUrl(callID);
  } else if(transf == 'conf') {
    CID = sessionParams.name;
    id = '';
    name = sessionParams.realName;
    email = sessionParams.name + '@' + domainName;
    url = '';
  }
  if(name === '') name = CID;
  var via = getCallVia(monitorData.call[callID]);
  var sel = $('#card');
  sel.html(
    '<a class="closeIcon right pointer" title="Close"></a>' +
    '<button style="position: absolute; bottom: 0px; left: 0px; font-size: 14px; padding: 5px; height: 29px; border-radius: 0px 5px 0px 0px;" class="button bg" data-value="' + id + '">' +
    translate('moreinfo') + '</button>' +
    '<span style="position: absolute; bottom: 0px; right: 0px; padding: 5px; background: none repeat scroll 0% 0% #CC6699; border-top-left-radius: 5px; color: rgb(34, 44, 54);">' +
    CID + '</span>' +
    '<p style="font-size: 30px; color: rgb(255, 255, 255);">' + name + '</p>' +
    '<p style="color: rgb(255, 255, 255); padding: 5px 0px;">' + email + '</p>' +
    (via ? '<p style="color: rgb(255, 255, 255); padding: 5px 0px;">' + via + '</p>' : '')
  );
  displayCallNoteForm(callID, getCallQueue(monitorData.call[callID]));
  sel.find('.button').on('click', function() {
    openClientIFrame(sessionParams.crmType, url);
  });
  if(peerID != callID || !callID) {
    getDirectoryData();
    if(!$('#transfClient').length) {
      $('body').append('<div id="transfClient"></div>');
    }
    $('#transfClient').html(
      '<input class="bg input" type="text" />' +
      (transf != 'transfer' ? '<button id="bHangup" class="button bg right">' + translate('hangup') + '</button>' : '') +
      '<button id="bCall" class="ff0 button right">' +
      (transf == 'transfer' ? translate('attended') : translate('call')) + '</button>'+
      (transf == 'transfer' ? '<button id="bBlind" class="button bg right">' + translate('blind') + '</button>' : '') +
      '<div id="directoryDataContainer"><div class="scrollList"><div id="directoryData"></div></div></div></div>'
    );
    $('#bHangup').on('click', function() {
      $('#transfClient').remove();
      $('#card').remove();
      $('#callNotes').remove();
      stopPeer(peerID);
    });
    $('#transfClient .input').on('keyup', function() {
      if(this.value.replace(/\s/g).length) {
        $('#directoryData .cn:Contains("' + this.value + '")').show();
        $('#directoryData .cn:not(.cn:Contains("' + this.value + '"))').hide();
        $('#directoryDataContainer > div').data('jsp').scrollToY(0);
      } else {
        $('#directoryData .cn').show();
      }
    });
    $('#bBlind').on('click', function() {
      var cld = $('#transfClient .input').val();
      if(!cld) {
        return;
      }
      $('#transfClient').remove();
      $('#card').remove();
      $('#callNotes').remove();
      $('#atransf').removeClass('ff0');
      makeTransfer(callID, cld, 'blind');
    });
    $('#bCall').on('click', function() {
      var cld = $('#transfClient .input').val();
      if(!cld) {
        return;
      }
      $('#transfClient').remove();
      $('#card').remove();
      $('#callNotes').remove();
      $('#atransf').removeClass('ff0');
      if(transf == 'transfer') {
        makeTransfer(callID, cld, 'attended');
      } else if(transf == 'conf') {
        var cldType = '';
        var cn = getDirectoryRecord(cld);
        if(cn) cldType = cn.type;
        if(callID) confAdd(callID, cld, cldType);
        else makeConf(sessionParams.name, cld, cldType);
        $('#aconf').removeClass('ff0');
      } else {
        redirectCall(callID, peerID, cld);
      }
    });
  }
  sel.find('.closeIcon').on('click', function() {
    $(this).parent().remove();
    $('#transfClient').remove();
    $('.turn a.hl').removeClass('hl');
    $('#atransf').removeClass('ff0');
    $('#aconf').removeClass('ff0');
    $('#callNotes').remove();
  });
}
function getCallVia(callObj) {
  if(!callObj) return null;
  var peers = Object.keys(callObj.peers);
  var iKey;
  for(var i = 0; i < peers.length; ++i) {
    iKey = peers[i];
    if(callObj.peers[iKey].isAgent) return callObj.peers[iKey].via;
  }
  return null;
}
function getCallQueue(callObj) {
  if(!callObj) return null;
  var peers = Object.keys(callObj.peers);
  var iKey;
  for(var i = 0; i < peers.length; ++i) {
    iKey = peers[i];
    if(callObj.peers[iKey].via) return callObj.peers[iKey].via;
    else if(callObj.peers[iKey].qName) return callObj.peers[iKey].qName;
  }
  return null;
}
function displayDirectoryData() {
  for(var i = 0; i < directoryData.length; i++) {
    if(directoryData[i].name == 'pbx') {
      continue;
    }
    $('#directoryData').append(
      '<p class="cn pointer' + (directoryData[i].type == 'group' ? ' group' : '') + '" data-value="' + directoryData[i].name + '">' +
      '<span>' + getTruncName(directoryData[i].name, 20) + '</span>' +
      '<span class="right RealName">' +
      (directoryData[i].RealName != 'none' ? getTruncName(directoryData[i].RealName, 20) : '') + '</span></p>'
    );
  }
  $('#directoryDataContainer .scrollList').jScrollPane();
  $('#directoryData .cn').on('click', function() {
    $('#transfClient .input').val($(this).attr('data-value'));
    //$('#transfClient .button').trigger('click');
  });
}
function getCallByPeer(peerID) {
  var calls = Object.keys(monitorData.call);
  var iKey;
  for(var i = 0; i < calls.length; ++i) {
    iKey = calls[i];
    if(monitorData.call[iKey].peers[peerID]) {
      return iKey;
    }
  }
  return null;
}
/*
function fixQItemsWidth(qName) {
  var t = $('#mon_' + qName);
  if(t.hasClass('active')) {
    var head = t.find('ul.head');
    var h = head.outerHeight() + head.siblings('.monAgentFilterContainer').outerHeight() + head.siblings('.items').outerHeight() + 52;
    t.stop().animate({'height':h},700);
  }
}
*/
