// ================================================== //
//  Contact Center XIMSS Controller interface module  //
//                   Version 2.2.5                    //
//                                                    //
// ---------- (c) AO SBK 2015-2022 ---------- //
// ================================================== //

function XIMSSController() {
  var that = this;
  var ximssSession = null;
  var monitorRefreshed = null;
  var delayRefresh = null;

  function ximssAsyncProcessor(xmlResponse) {
    if(!sessionId) {
			sessionId = xmlResponse.getAttribute('urlID');
      userName = xmlResponse.getAttribute('userName');
      if(userName && userName.indexOf('@') >= 0) {
        domainName = userName.substring(userName.indexOf('@') + 1);
      }
		}
    if(xmlResponse.tagName == 'errorIM') {
      clearTimeout(addSessionTimer);
      addSessionTimer = null;
      checkCcTask();
    }
    if(xmlResponse.tagName == 'bye') {
      logoutContactCenter();
    }
    if(xmlResponse.getAttribute('eventName') == 'rspSetAgentStatus') {
      var result = $.parseJSON(xmlResponse.textContent);
      if(result.error) alert(result.error);
    }
  }
  function ximssEventAddSession(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      logoutContactCenter();
      return;
    }
    if(!taskRef) {
      taskRef = xmlResponse.getAttribute('taskRef');
    }
    clearTimeout(addSessionTimer);
    addSessionTimer = null;
    sessionParams = result;
    // actions to do after session is set up
    refreshMonitor();
    if(sessionParams.access.admin.indexOf('anyOne') >= 0) {
      getSettings('general');
    }
    if(sessionParams.access.admin.length) {
      getGroups();
    }
    if(sessionParams.isDomainAdmin) {
      getUserDefaults();
    }
    if(sessionParams.iFrameName) {
      $('#iFrameLink span').text(sessionParams.iFrameName);
    }
    //getStatistics();
    getAgents();
    showInterface();
    closeOverlay();
  }
  function ximssEventGetSettings(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      closeOverlay();
    }
    if(result.type) {
      if(result.type == 'ivr') {
        displayIvrSettings(result);
      } else {
        displaySettings(result);
      }
    }
  }
  function ximssEventUpdateSettings(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      closeOverlay();
    }
    if(result.type == 'group') {
      getGroups();
    }
    if(result.type) {
      if(result.type == 'ivr') {
        if(result.name != ivrTableName) {
          getIvrs();
        }
      } else if(result.type == 'smsTemplate') {
        if(result.name != smsTableName) {
          getSMSTemplates();
        }
      } else {
        displaySettings(result);
      }
    }
  }
  function ximssEventGetGroups(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.objects) {
      displayGroupList(result.objects);
    }
  }
  function ximssEventGetAgents(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.objects) {
      displayAgentList(result.objects);
    }
    if(isArrElem(sessionParams.name, agents)) {
      $('#agentPanel').show();
      if(sessionParams.iFrameName) {
        if($('#tabCard').hasClass('tabActive')) $('#iFrameLink').css('display', 'inline');
        if(sessionParams.crmType && sessionParams.crmServer) {
          authIFrame(sessionParams.crmType, sessionParams.crmServer, sessionParams.name, passwd);
        } else if(sessionParams.iFrameUrl) {
          $('#iFrame').attr('src', sessionParams.iFrameUrl);
        }
      }
      $('.footer').hide();
    } else {
      $('#agentPanel').hide();
      $('#iFrameLink').hide();
      $('#tabMonitors iframe').hide();
    }
    gotAgents();
  }
  function ximssEventDisplayObjects(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.length) {
      updateMonitor(result);
    }
  }
  function ximssEventRemoveAgentFromGroup(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      closeOverlay();
    }
    else {
      getSettings('agent', result.aName);
      getSettings('group', result.gName);
      getGroups();
    }
  }
  function ximssEventCreateObject(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      closeOverlay();
    } else if(result.type != 'ivr' || result.type != 'smsTemplate') {
      objCreated();
    }
    if(result.type == 'ivr') {
      getIvrs();
    } else if(result.type == 'smsTemplate') {
      getSMSTemplates();
    }
  }
  function ximssEventRemoveObject(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result && result.error) {
      alert(result.error);
      closeOverlay();
    } else {
      objRemoved();
    }
    if(result.type == 'group') {
      getGroups();
    } else if(result.type == 'ivr') {
      getIvrs();
    } else if(result.type == 'smsTemplate') {
      getSMSTemplates();
    }
  }
  function ximssEventDoRefresh(xmlResponse) {
    if(monitorRefreshed) {
      monitorRefreshed = null;
      resetMonitor();
    } else {
      delayRefresh = true;
    }
  }
  function ximssEventRelogin(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    alert(translate(['reloginReasons', result.reason]));
    if(result.reason == 'agentRemoved' || result.reason == 'strictMode') {
      logoutContactCenter();
      return;
    }
    var sid = sessionId;
    var username = $('#login').val();
    createOverlay();
    clearData();
    //meetingName = '';
    taskRef = null;
    sessionParams = null;
    resetMonitorData();
    groups = [];
    agents = [];
    statistics = {};
    timeframe = {
  	  firstDate: '',
    	secondDate: '',
    	firstHour: 0,
    	secondHour: 24
    };
    recordDefault = null;
    createMeeting();
  }
  function ximssEventRefreshObjects(xmlResponse) {
    if(delayRefresh) {
      delayRefresh = null;
      resetMonitor();
    } else {
      monitorRefreshed = true;
      getStatistics();
    }
  }
  function ximssEventTransferCall(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result && result.error) {
      alert(result.error);
    }
  }
  function ximssEventAgentListener(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result && result.error) {
      alert(result.error);
    }
  }
  function ximssEventCallListener(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result && result.error) {
      alert(result.error);
    }
  }
  function ximssEventGetIvrs(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.objects) {
      displayIvrList(result.objects);
    } else {
      displayIvrList([]);
    }
  }
  function ximssEventGetSMSTemplates(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.objects) {
      displaySMSTemplatesList(result.objects);
    } else {
      displaySMSTemplatesList([]);
    }
  }
  function ximssEventStatsProgress(xmlResponse) {
    var progress = $.parseJSON(xmlResponse.textContent);
    displayStatsProgress(progress.percent);
  }
  function ximssEventStatsDone(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
      displayStatsProgress(null);
    } else {
      displayStatsProgress(result.reportName);
    }
  }
  function ximssEventGetStatistics(xmlResponse) {
    var result = $.parseJSON(xmlResponse.textContent);
    if(result.error) {
      alert(result.error);
    }
  }
  function ximssNetworkErrorProcessor(isFatal,timeElapsed) {
    if(langFlag) {
      langFlag = false;
      return true;
    }
    console.log(isFatal ? 'fatal' : 'having problems for ' + timeElapsed + ' sec');
    if(isFatal || parseInt(timeElapsed) > 60) {
      alert(translate('connectionError'));
      location.href = 'cc.html';
    } else {
      $('#reconnecting').show();
      var timerId = $('#reconnecting').attr('data-value');
      if(timerId) {
        clearTimeout(timerId);
      }
      timerId = setTimeout(function() {
        $('#reconnecting').hide();
        $('#reconnecting').removeAttr('data-value');
      }, 30000);
      $('#reconnecting').attr('data-value', timerId);
    }
  }
  function loginCallback(newSession,errorCode) {
    if(errorCode != null) {
      that = null;
      alert(errorCode);
      closeOverlay();
      return;
    }
    ximssSession = newSession;
    ximssSession.setUnknownAsyncProcessor(that,ximssAsyncProcessor);
    ximssSession.setNetworkErrorProcessor(that,ximssNetworkErrorProcessor);
    ximssSession.setAsyncProcessor(that,ximssEventAddSession,'taskEvent','eventName','rspAddSession');
    ximssSession.setAsyncProcessor(that,ximssEventGetSettings,'taskEvent','eventName','rspGetSettings');
    ximssSession.setAsyncProcessor(that,ximssEventUpdateSettings,'taskEvent','eventName','rspUpdateSettings');
    ximssSession.setAsyncProcessor(that,ximssEventGetGroups,'taskEvent','eventName','rspGetGroups');
    ximssSession.setAsyncProcessor(that,ximssEventGetAgents,'taskEvent','eventName','rspGetAgents');
    ximssSession.setAsyncProcessor(that,ximssEventDisplayObjects,'taskEvent','eventName','monDisplayObjects');
    ximssSession.setAsyncProcessor(that,ximssEventRemoveAgentFromGroup,'taskEvent','eventName','rspRemoveAgentFromGroup');
    ximssSession.setAsyncProcessor(that,ximssEventCreateObject,'taskEvent','eventName','rspCreateObject');
    ximssSession.setAsyncProcessor(that,ximssEventRemoveObject,'taskEvent','eventName','rspRemoveObject');
    ximssSession.setAsyncProcessor(that,ximssEventRefreshObjects,'taskEvent','eventName','rspRefreshObjects');
    ximssSession.setAsyncProcessor(that,ximssEventDoRefresh,'taskEvent','eventName','monDoRefresh');
    ximssSession.setAsyncProcessor(that,ximssEventTransferCall,'taskEvent','eventName','rspTransferCall');
    ximssSession.setAsyncProcessor(that,ximssEventRelogin,'taskEvent','eventName','monRelogin');
    ximssSession.setAsyncProcessor(that,ximssEventAgentListener,'taskEvent','eventName','rspSetAgentListener');
    ximssSession.setAsyncProcessor(that,ximssEventCallListener,'taskEvent','eventName','rspSetCallListener');
    ximssSession.setAsyncProcessor(that,ximssEventGetIvrs,'taskEvent','eventName','rspGetIVRs');
    ximssSession.setAsyncProcessor(that,ximssEventGetSMSTemplates,'taskEvent','eventName','rspGetSMSTemplates');
    ximssSession.setAsyncProcessor(that,ximssEventGetStatistics,'taskEvent','eventName','rspGetStatistics');
    ximssSession.setAsyncProcessor(that,ximssEventStatsProgress,'taskEvent','eventName','monStatsProgress');
    ximssSession.setAsyncProcessor(that,ximssEventStatsDone,'taskEvent','eventName','monStatsDone');
    ximssSession.start();
    
    checkAccountLanguage();

    createMeeting();
    return;
  }
  function logoutCallback(errorCode,xmlRequest) {
    hideInterface();
  }
  function dataCallback(xmlData,xmlRequest) {
    if(xmlRequest.tagName == 'cliExecute') {
      if(xmlRequest.textContent.indexOf('LISTACCOUNTS') == 0) {
        var list = [];
        for(var i = 0; i < xmlData.childNodes.length; i++) {
          list.push(xmlData.childNodes[i].getAttribute('key'));
        }
        displayUserList(list);
        displayUserListPstn(list);
        return;
      }
      if(xmlRequest.textContent.indexOf('GETACCOUNTSETTINGS') == 0) {
        var rn;
        var r;
        var pstn = {};
        var rsip = {};
        var am = undefined;
        for(var i = 0; i < xmlData.childNodes.length; i++) {
          if(xmlData.childNodes[i].getAttribute('key') == 'RealName') {
            rn = xmlData.childNodes[i].textContent;
          }
          if(xmlData.childNodes[i].getAttribute('key') == 'PSTNRecord') {
            if(xmlData.childNodes[i].textContent == 'YES') {
              r = 'Yes';
            } else if(xmlData.childNodes[i].textContent == '#NULL#') {
              r = '';
            } else {
              r = 'No';
            }
          }
          if(xmlData.childNodes[i].getAttribute('key') && xmlData.childNodes[i].getAttribute('key').indexOf('PSTN') == 0) {
            pstn[xmlData.childNodes[i].getAttribute('key')] = xmlData.childNodes[i].textContent;
          }
          if(xmlData.childNodes[i].getAttribute('key') == 'RSIP') {
            for(var j = 0; j < xmlData.childNodes[i].childNodes.length; j++) {
              var name = xmlData.childNodes[i].childNodes[j].getAttribute('key');
              rsip[name] = {};
              for(var k = 0; k < xmlData.childNodes[i].childNodes[j].childNodes.length; k++) {
                rsip[name][xmlData.childNodes[i].childNodes[j].childNodes[k].getAttribute('key')] = xmlData.childNodes[i].childNodes[j].childNodes[k].textContent;
              }
            }
          }
          if(xmlData.childNodes[i].getAttribute('key') == 'AccessModes') {
            am = [];
            if(xmlData.childNodes[i].textContent == 'All') {
              am.push('All');
            } else {
              var amNode = xmlData.childNodes[i];
              for(var j = 0; j < amNode.childNodes.length; j++) {
                if(amNode.childNodes[j].tagName == 'subValue') {
                  am.push(amNode.childNodes[j].textContent);
                }
              }
            }
          }
        }
        displayUserInfo({ RealName: rn, PSTNRecord: r, AccessModes: am });
        var name = xmlRequest.textContent.split(' ')[1];
        displayUserInfoPstn(name, pstn, rsip);
        return;
      }
      if(xmlRequest.textContent.indexOf('GETACCOUNTALIASES') == 0) {
        var a = [];
        for(var i = 0; i < xmlData.childNodes.length; i++) {
          a.push(xmlData.childNodes[i].textContent);
        }
        displayUserInfo({ aliases: a.join(',') });
        return;
      }
      if(xmlRequest.textContent.indexOf('GETACCOUNTDEFAULTS') == 0) {
        var r;
        generalPstn = {};
        for(var i = 0; i < xmlData.childNodes.length; i++) {
          if(xmlData.childNodes[i].getAttribute('key') == 'PSTNRecord') {
            r = xmlData.childNodes[i].textContent;
          }
          if(xmlData.childNodes[i].getAttribute('key') && xmlData.childNodes[i].getAttribute('key').indexOf('PSTN') == 0) {
            generalPstn[xmlData.childNodes[i].getAttribute('key')] = xmlData.childNodes[i].textContent;
          }
          if(xmlData.childNodes[i].getAttribute('key') == 'AccessModes') {
            accountDefaults['AccessModes'] = [];
            if(xmlData.childNodes[i].textContent == 'All') {
              accountDefaults['AccessModes'].push('All');
            } else {
              var amNode = xmlData.childNodes[i];
              for(var j = 0; j < amNode.childNodes.length; j++) {
                if(amNode.childNodes[j].tagName == 'subValue') {
                  accountDefaults['AccessModes'].push(amNode.childNodes[j].textContent);
                }
              }
            }
          }
        }
        if(!r || r == 'NO' || r != 'YES') {
          recordDefault = 'No';
        } else {
          recordDefault = 'Yes';
        }
        displayGeneralPstn();
        listUsers();
      }
      if(xmlRequest.textContent.indexOf('LISTDOMAINPBXFILES') == 0) {
        var result = new Array();
        for(var i = 0; i < xmlData.childNodes.length; i++) {
          var fileName = xmlData.childNodes[i].getAttribute('key');
          if(fileName && /\.wav$/i.test(fileName)) {
            result.push(fileName);
          }
        }
        listSoundFiles(result);
      }
    } else if(xmlRequest.tagName == 'listDirectory') {
      var contact = {};
      for(var i = 0; i < xmlData.childNodes.length; i++) {
        if(xmlData.childNodes[i].getAttribute('name') == 'cn') {
          contact.RealName = xmlData.childNodes[i].textContent;
        } else if(xmlData.childNodes[i].getAttribute('name') == 'uid') {
          contact.name = xmlData.childNodes[i].textContent;
        } else if(xmlData.childNodes[i].getAttribute('name') == 'objectclass') {
          if(xmlData.childNodes[i].textContent == 'CommuniGateAccount') {
            contact.type = 'account';
          } else if(xmlData.childNodes[i].textContent == 'CommuniGateGroup') {
            contact.type = 'group';
          } else {
            return;
          }
        }
      }
      directoryData.push(contact);
    } else if(xmlRequest.tagName == 'bannerRead') {
      var data = xmlData.textContent.replace(/\n/g, '<endline>');
      var response = $.parseJSON(data);
      if(response.error) {
        if(response.op == 'addTel' && /UNIQUE constraint failed/.test(response.error)) {
          updExistingTel();
        } else if(response.op == 'addEmail' && /UNIQUE constraint failed/.test(response.error)) {
          updExistingEmail();
        } else {
          alert(response.error);
        }
        return;
      }
      if(response.op == 'addCard') {
        $('#card_info .loader').hide().siblings('#cgc').show();
        if(response.result && response.result.uid) {
          cg_sel.find('.card_uid').val(response.result.uid).trigger('change');
        }
        return;
      } else if(response.op == 'updateCard') {
        $('#card_info .loader').hide().siblings('#cgc').show();
        cg_sel.show().siblings().hide();
      } else if(response.op == 'getCardByNum' || response.op == 'getCardByEmail' || response.op == 'getCardsByName' || response.op == 'getCardByUid') {
        $('#card_info .loader').hide().siblings('#cgc').show();
        if('length' in response.result) {
          if(response.result.length == 1) {
            displayCGCard({cgcard:response.result[0]}, $('#cgc_found'));
          } else {
            displayCGSearchRL(response.result);
          }
        } else {
          displayCGCard({cgcard:response.result}, $('#cgc_found'));
        }
      } else if(response.op == 'getCallsByCardUid') {
        //$('#card_info .loader').hide().siblings('#cgh').show();
        displayCGCalls(response.result)
      } else if(response.op == 'addTel' || response.op == 'updateTel') {
        postAddTel();
      } else if(response.op == 'addEmail' || response.op == 'updateEmail') {
        postAddEmail();
      }
    } else if(xmlRequest.tagName == 'fileList' && xmlRequest.getAttribute('directory') == script_dir) {
      if(xmlData.getAttribute('type') != 'directory') {
        script_list.push(xmlData.getAttribute('fileName'));
      }
    } else if(xmlRequest.tagName == 'fileRead' && RegExp(script_dir).test(xmlRequest.getAttribute('fileName'))) {
      displayScript(xmlRequest.getAttribute('fileName'), xmlData.textContent);
    } else if(xmlRequest.tagName == 'prefsRead') {
      for(var i = 0; i < xmlData.childNodes.length; i++) {
        if(xmlData.childNodes[i].nodeName == 'Language') {
          $('.ilang').siblings('.description').text($('.ilang').find('option[value="' + xmlData.childNodes[i].textContent + '"]').text());
          getStrings(xmlData.childNodes[i].textContent);
          break;
        }
      }
    } else if(xmlRequest.tagName == 'httpCall') {
      if(xmlData.tagName == 'retrievedURL' && xmlData.getAttribute('responseCode') == '200') {
        compareVersion(xmlData.textContent);
      }
    }
  }
  function finalCallback(errorCode,xmlRequest) {
    if(errorCode != null) {
      if(xmlRequest.tagName == 'cliExecute') {
        if(xmlRequest.textContent.indexOf('GETNAMEDTASK') == 0) {
          createCcTask();
        } else if(xmlRequest.textContent.indexOf('STOREDOMAINPBXFILE') == 0 && errorCode == 'file is not found') {
          that.sendRequest(
            'cliExecute',
            null,
            'CREATEDOMAINPBX ' + domainName
          );
          that.sendRequest('cliExecute', null, xmlRequest.textContent);
        } else {
          alert(errorCode);
          if(xmlRequest.textContent.indexOf('UPDATEACCOUNTDEFAULTS') == 0) {
            listUsers();
            displayGeneralPstn();
          }
          closeOverlay();
        }
      }
      if(errorCode == 'async Object not found') {
        closeSession();
        alert('no response from ' + ccTask + '@' + domainName);
        location.reload();
      }
    } else {
      if(xmlRequest.tagName == 'taskCreateMeeting') activateMeeting();
      if(xmlRequest.tagName == 'taskActivateMeeting') addCcSession();
      if(xmlRequest.tagName == 'cliExecute') {
        if(xmlRequest.textContent.indexOf('GETNAMEDTASK') == 0) {
          closeSession();
          alert('no response from ' + ccTask + '@' + domainName);
        } else if(xmlRequest.textContent.indexOf('CREATENAMEDTASK') == 0) {
          setCcTaskDefaults();
        } else if(xmlRequest.textContent.indexOf('UPDATENAMEDTASK') == 0) {
          addCcSession();
        } else if(xmlRequest.textContent.indexOf('UPDATEACCOUNTSETTINGS') == 0) {
          listUsers();
        } else if(xmlRequest.textContent.indexOf('CREATEACCOUNT') == 0) {
          userCreated();
        } else if(xmlRequest.textContent.indexOf('DELETEACCOUNT') == 0) {
          userRemoved();
        } else if(xmlRequest.textContent.indexOf('UPDATEACCOUNTDEFAULTS') == 0) {
          getUserDefaults();
        } else if(xmlRequest.textContent.indexOf('STOREDOMAINPBXFILE') == 0) {
          getSoundFiles($('#pbxEnv').val());
        }
      }
      if(xmlRequest.tagName == 'fileStore') {
        $(document).trigger('fileStored');
      }
      if(xmlRequest.tagName == 'fileRemove') {
        $(document).trigger('fileRemoved');
      }
      if(xmlRequest.tagName == 'listDirectory') {
        displayDirectoryData();
      }
      if(xmlRequest.tagName == 'fileList' && xmlRequest.getAttribute('directory') == script_dir) {
        displayScriptList();
      }
    }
  }

  this.createSession = function(serverAddress,userName,password,sessionIDMethod,killOld) {
    var params = new Object;
    params.binding = 'HTTP';
    if(location.protocol == 'https:') params.secureMode = 'YES';
    params.serverName = serverAddress;
    if(sessionIDMethod) {
      params.loginMethod = 'sessionid';
      params.sessionid = password;
      if(killOld) {
        params.killOldSession = true;
      }
    } else {
      params.loginMethod = 'auto';
      params.password = password;
    }
    params.userName = userName;
    params.pollPeriod = 20;
    new XIMSSSession(params,this,loginCallback);
    return;
  }
  this.closeSession = function() {
    if(ximssSession != null) {
      ximssSession.close(that,logoutCallback);
    }
  }
  this.sendRequest = function(request,params,data) {
    if(ximssSession == null) {
      alert('ximssSession does not exist');
      return;
    }
    xmlRequest = ximssSession.createXMLNode(request);
    if(request == 'cliExecute') {
      xmlRequest.setAttribute('report','xml');
    }
    if(params != null) {
      var keys = Object.keys(params);
      for(var i = 0; i < keys.length; ++i) {
        xmlRequest.setAttribute(keys[i],params[keys[i]]);
      }
    }
    if(data != null) {
      if(typeof(data) == 'object') {
        xmlRequest = createXMLSubKeys(xmlRequest, data);
      } else {
        xmlRequest.appendChild(ximssSession.createTextNode(data));
      }
    }
    ximssSession.sendRequest(xmlRequest,that,dataCallback,finalCallback,true);
  }
  this.storeFile = function(uploadID, fN) {
    if(ximssSession == null) return;
    xmlRequest = ximssSession.createXMLNode("fileStore");
    xmlRequest.setAttribute("fileName",fN);
    xmlRequest.setAttribute("uploadID",uploadID);
    ximssSession.sendRequest(xmlRequest,that,null,finalCallback,true);
  }
  function createXMLSubKeys(node, data) {
    if(typeof(data) == 'object') {
      if(getKeysCount(data)) {
        var keys = Object.keys(data);
        var key;
        for(var i = 0; i < keys.length; ++i) {
          key = keys[i];
          var element;
          if(isNaN(+key) || key == '' || !('length' in data)) {
            element = ximssSession.createXMLNode('subKey');
            element.setAttribute('key', key);
          } else {
            element = ximssSession.createXMLNode('subValue');
          }
          var value;
          if(typeof(data[key]) == 'object') {
            element = createXMLSubKeys(element, data[key]);
          } else {
            value = ximssSession.createTextNode(data[key]);
            element.appendChild(value);
          }
          if(data[key] !== '' || (typeof(data[key]) == 'object' && !getKeysCount(data[key]))) {
            node.appendChild(element);
          }
        }
        if(!node.childNodes.length && !('length' in data)) {
          node.appendChild(ximssSession.createXMLNode('subKey'));
        }
      } else {
        if('length' in data) {
          node.appendChild(ximssSession.createXMLNode('subValue'));
        } else {
          node.appendChild(ximssSession.createXMLNode('subKey'));
        }
      }
    } else {
      node.appendChild(ximssSession.createTextNode(data));
    }
    return node;
  }
  function parseXML(xml) {
    var result;
    var type;
    if(xml.nodeType == 1 || xml.nodeType == 9) {
      if(xml.childNodes.length) {
        if(xml.childNodes[0].nodeName == 'subKey') {
          // dict
          result = {};
          type = 'dict';
        } else if(xml.childNodes[0].nodeName == 'subValue') {
          // array
          result = [];
          type = 'array';
        } else {
          // other - assume we have only a text value
          return getXMLValue(xml.childNodes[0]);
        }
        for(var i = 0; i < xml.childNodes.length; i++) {
          if(xml.childNodes[i].nodeType == 1) {
            if(type == 'dict') {
              result[xml.childNodes[i].getAttribute('key')] = parseXML(xml.childNodes[i]);
            } else if(type == 'array') {
              result.push(parseXML(xml.childNodes[i]));
            }
          } else if(xml.childNodes[i].nodeType == 3) {
            if(type == 'dict') {
              result[xml.childNodes[i].getAttribute('key')] = getXMLValue(xml.childNodes[i]);
            } else if(type == 'array') {
              result.push(getXMLValue(xml.childNodes[i]));
            }
          }
        }
        return result;
      } else {
        return null;
      }
    } else if(xml.nodeType == 3) {
      // text
      return getXMLValue(xml);
    } else {
      return undefined;
    }
  }
  function getXMLValue(xml) {
    if(xml.textContent == '#NULL#') {
      return null;
    } else if(xml.textContent == 'YES') {
      return true;
    } else if(xml.textContent == 'NO') {
      return false;
    } else {
      return xml.textContent;
    }
  }
}

function createSession(serverName,serverPort,userName,password,sessionIDMethod,killOld) {
  if(myController != null) {
    myController = null;
    resetVars();
  }
  var serverAddress = serverName;
  if(serverPort != null) serverAddress += ':' + serverPort;
  myController = new XIMSSController();
  myController.createSession(serverAddress,userName,password,sessionIDMethod,killOld);
}
function closeSession() {
  if(myController != null) {
    if(taskRef) {
      myController.sendRequest('taskSendEvent', { eventName: 'sessBye', taskRef: taskRef });
    }
    var params = {
      meetingSet: meetingSet,
      meetingName: meetingName,
    };
    myController.sendRequest('taskRemoveMeeting', params);
    meetingName = '';
    myController.closeSession();
  }
}
function createMeeting() {
  if(myController != null) {
    if(meetingName) {
      var params = {
        meetingSet: meetingSet,
        meetingName: meetingName,
      };
      myController.sendRequest('taskRemoveMeeting', params);
    }
    meetingName = (Math.floor( Math.random() * 900000 ) + 100000).toString();
    var params = {
      meetingSet: meetingSet,
      meetingName: meetingName,
    };
    myController.sendRequest('taskCreateMeeting', params);
  }
}
function activateMeeting() {
  if(myController != null) {
    var params = {
      meetingSet: meetingSet,
      meetingName: meetingName,
    };
    myController.sendRequest('taskActivateMeeting', params);
  }
}
function addCcSession() {
  if(myController != null) {
    var params = {
      peer: ccTask + '@' + domainName,
    };
    var data = '{cmd=sessAddSession;setName=' + meetingSet + ';key=' + meetingName + ';shortId=' + sessionId.substr(0,sessionId.indexOf('-')) + ';}';
    myController.sendRequest('sendIM', params, data);
    addSessionTimer = setTimeout(function(){
      createCcTask();
    }, addSessionTimeout);
  }
}
function checkCcTask() {
  if(myController != null) {
    var cli = 'GETNAMEDTASK ' + ccTask + '@' + domainName;
    myController.sendRequest('cliExecute', null, cli);
  }
}
function createCcTask() {
  if(myController != null) {
    var cli = 'CREATENAMEDTASK ' + ccTask + '@' + domainName + ' FOR ' + taskOwner;
    myController.sendRequest('cliExecute', null, cli);
  }
}
function setCcTaskDefaults() {
  if(myController != null) {
    var cli = 'UPDATENAMEDTASK ' + ccTask + '@' + domainName + ' ' + defaultSettings;
    myController.sendRequest('cliExecute', null, cli);
  }
}

function getSettings(type, name) {
  if(myController != null && type) {
    var body = {};
    body.type = type;
    if(name) body.name = name;
    myController.sendRequest(
      'taskSendEvent',
      { eventName: 'sessGetSettings', taskRef: taskRef },
      body
    );
  }
}

function updateSettings(type, name, settings) {
  if(myController != null && type) {
    var body = {};
    body.type = type;
    if(name) body.name = name;
    body.settings = settings;
    myController.sendRequest(
      'taskSendEvent',
      { eventName: 'sessUpdateSettings', taskRef: taskRef },
      body
    );
  }
}

function getGroups() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      { eventName: 'sessGetGroups', taskRef: taskRef }
    );
  }
}

function getUserDefaults() {
  if(myController != null) {
    myController.sendRequest('cliExecute', null, 'GETACCOUNTDEFAULTS ' + domainName);
  }
}

function updateUserDefaults(params) {
  if(myController != null) {
    myController.sendRequest(
      'cliExecute',
      null,
      'UPDATEACCOUNTDEFAULTS ' + domainName + ' ' + makeCGPDict(params)
    );
  }
}

function getAgents() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      { eventName: 'sessGetAgents', taskRef: taskRef }
    );
  }
}

function getIvrs() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      { eventName: 'sessGetIVRs', taskRef: taskRef }
    );
  }
}

function getUsers() {
  if(myController != null) {
    myController.sendRequest('cliExecute', null, 'LISTACCOUNTS ' + domainName);
  }
}

function getUserInfo(name) {
  if(myController != null) {
    myController.sendRequest('cliExecute', null, 'GETACCOUNTSETTINGS ' + name);
    myController.sendRequest('cliExecute', null, 'GETACCOUNTALIASES ' + name);
  }
}

function createUser(name, userInfo) {
  if(myController != null) {
    if(name) {
      myController.sendRequest(
        'cliExecute',
        null,
        'CREATEACCOUNT ' + name + (userInfo ? ' {' +
        (userInfo.RealName ? 'RealName="' + userInfo.RealName + '";' : '') +
        (userInfo.Password ? 'Password="' + userInfo.Password + '";' : '') +
        (userInfo.PSTNRecord ? 'PSTNRecord="' + userInfo.PSTNRecord + '";' : '') +
        '}' : '')
      );
    }
  }
}

function removeUser(name) {
  if(myController != null) {
    if(name) {
      myController.sendRequest(
        'cliExecute',
        null,
        'DELETEACCOUNT ' + name
      );
    }
  }
}

function updateUserInfo(name, userInfo) {
  if(myController != null) {
    if(name) {
      if(name.indexOf('@') < 0) name += '@' + domainName;
      if(userInfo.Password) {
        myController.sendRequest(
          'cliExecute',
          null,
          'SETACCOUNTPASSWORD "' + name + '" PASSWORD "' + userInfo.Password + '"'
        );
        delete userInfo.Password;
      }
      myController.sendRequest(
        'cliExecute',
        null,
        'UPDATEACCOUNTSETTINGS ' + name + ' ' + makeCGPDict(userInfo)
      );
    }
  }
}

function updateUserRsips(name, rsip) {
  if(myController != null) {
    if(name) {
      if(name.indexOf('@') < 0) name += '@' + domainName;
      myController.sendRequest(
        'cliExecute',
        null,
        'SETACCOUNTRSIPS ' + name + ' ' + makeCGPDict(rsip)
      );
    }
  }
}

function updateUserAliases(name, aliases) {
  if(myController != null) {
    if(name) {
      if(name.indexOf('@') < 0) name += '@' + domainName;
      myController.sendRequest(
        'cliExecute',
        null,
        'SETACCOUNTALIASES ' + name + ' (' + aliases.join(',') + ')'
      );
    }
  }
}

function removeAgentFromGroup(aName, gName) {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessRemoveAgentFromGroup',
        taskRef: taskRef,
      },
      {
        aName: aName,
        gName: gName,
      }
    );
  }
}

function createObject(type, name, settings, fileRef) {
  if(myController != null && type && name && settings) {
    var body = {};
    body.type = type;
    body.name = name;
    body.settings = settings;
    if(fileRef) {
      body.fileRef = fileRef;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessCreateObject',
        taskRef: taskRef,
      },
      body
    );
  }
}

function removeObject(type, name) {
  if(myController != null && type && name) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessRemoveObject',
        taskRef: taskRef,
      },
      {
        type: type,
        name: name,
      }
    );
  }
}

function refreshMonitor() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessRefreshObjects',
        taskRef: taskRef,
      }
    );
  }
}

function setAgentStatus(name, status, statusInfo) {
  if(myController != null) {
    var params = {
        name: name,
        status: status,
    };
    if(statusInfo) params.statusInfo = statusInfo;
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessSetAgentStatus',
        taskRef: taskRef,
      },
      params
    );
  }
}

function setCampaignStatus(name, status) {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessSetCampaignStatus',
        taskRef: taskRef,
      },
      {
        name: name,
        status: status,
      }
    );
  }
}

function makeTransfer(callID, cld, mode) {
  if(myController != null) {
    var peerID = getMyPeerId(callID);
    if(!peerID) {
      return;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessTransferCall',
        taskRef: taskRef,
      },
      {
        name: callID,
        refPeer: peerID,
        referTo: cld,
        mode: mode,
      }
    );
  }
}

function redirectCall(callID, peerID, cld) {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessRedirectCall',
        taskRef: taskRef,
      },
      {
        name: callID,
        redirPeer: peerID,
        redirectTo: cld,
      }
    );
  }
}

function setAgentListener(agent, mode) {
  if(myController != null) {
    if(agent && mode) {
      myController.sendRequest(
        'taskSendEvent',
        {
          eventName: 'sessSetAgentListener',
          taskRef: taskRef,
        },
        {
          name: agent,
          mode: mode,
        }
      );
    }
  }
}

function setCallListener(callID, mode) {
  if(myController != null) {
    if(callID) {
      var data = {
        name: callID,
      };
      if(mode) data.mode = mode;
      myController.sendRequest(
        'taskSendEvent',
        {
          eventName: 'sessSetCallListener',
          taskRef: taskRef,
        },
        data
      );
    }
  }
}

function getSoundFiles(lang) {
  if(myController != null) {
    myController.sendRequest(
      'cliExecute',
      null,
      'LISTDOMAINPBXFILES ' + domainName + (lang ? ' FILE ' + lang : '')
    );
  }
}

function uploadSoundFile(fileName, fileData) {
  if(myController != null) {
    myController.sendRequest(
      'cliExecute',
      null,
      'STOREDOMAINPBXFILE ' + domainName + ' FILE "' + fileName + '" DATA "[' + fileData + ']"'
    );
  }
}
function setHold(callID, mode) {
  if(myController != null) {
    var peerID = getMyPeerId(callID);
    if(!peerID) {
      return;
    }
    var param = {
      pName: peerID
    };
    if(mode) {
      param.mode = mode;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessHold',
        taskRef: taskRef,
      },
      param
    );
  }
}
function hangUp(callID) {
  if(myController != null) {
    var peerID = getMyPeerId(callID);
    if(!peerID) {
      return;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessStopPeer',
        taskRef: taskRef,
      },
      {
        pName: peerID,
      }
    );
  }
}
function stopPeer(peerID) {
  if(myController != null && peerID != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessStopPeer',
        taskRef: taskRef,
      },
      { pName: peerID }
    );
  }
}
function setMute(callID, mode) {
  if(myController != null) {
    var peerID = getMyPeerId(callID);
    if(!peerID || !mode) {
      return;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessMute',
        taskRef: taskRef,
      },
      {
        pName: peerID,
        mode: mode,
      }
    );
  }
}
function stopCC() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessStopMonitor',
        taskRef: taskRef,
      },
      null
    );
    closeSession();
    location.reload();
  }
}
function getDirectoryData() {
  if(myController != null) {
    directoryData = [];
    myController.sendRequest(
      'listDirectory',
      {
        limit: 1000,
      }
    );
  }
}
function addTel(num, card_uid) {
  if(card_uid === '') card_uid = 'NULL';
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'addTel',
        param: { num: num, card_uid: card_uid },
      })
    );
  }
}
function updateTel(num, card_uid) {
  if(card_uid === '') card_uid = 'NULL';
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'updateTel',
        param: {
          num: num,
          card_uid: card_uid,
        },
      })
    );
  }
}
function refuseTel(num) {
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'updateTel',
        param: {
          num: num,
          card_uid: 'NULL',
        },
      })
    );
  }
}
function addEmail(email, card_uid) {
  if(card_uid === '') card_uid = 'NULL';
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'addEmail',
        param: { email: email, card_uid: card_uid },
      })
    );
  }
}
function updateEmail(email, card_uid) {
  if(card_uid === '') card_uid = 'NULL';
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'updateEmail',
        param: {
          email: email,
          card_uid: card_uid,
        },
      })
    );
  }
}
function refuseEmail(email) {
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'updateEmail',
        param: {
          email: email,
          card_uid: 'NULL',
        },
      })
    );
  }
}
function addCGCard(cgc) {
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'addCard',
        param: cgc,
      })
    );
  }
}
function updateCGCard(cgc) {
  if(myController != null) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'updateCard',
        param: cgc,
      })
    );
  }
}
function confAdd(callID, num, type) {
  if(myController != null) {
    var peerID = getMyPeerId(callID);
    if(!peerID || !num) {
      return;
    }
    myController.sendRequest(
      'taskSendEvent',
      {
        eventName: 'sessConferenceAdd',
        taskRef: taskRef,
      },
      {
        pName: peerID,
        partyAddr: num,
        partyType: type,
      }
    );
  }
}
function cgSearch(num, name, email) {
  if(myController != null) {
    var op;
    var qParam;
    if(num) {
      op = 'getCardByNum';
      qParam = num;
    } else if(email) {
      op = 'getCardByEmail';
      qParam = email;
    } else if(name) {
      op = 'getCardsByName';
      qParam = name;
    } else {
      return;
    }
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: op,
        param: qParam,
      })
    );
  }
}
function getCGCard(uid) {
  if(myController != null && uid) {
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify({
        op: 'getCardByUid',
        param: uid,
      })
    );
  }
}
function getScriptList() {
  if(myController != null) {
    myController.sendRequest(
      'fileList',
      {
        directory: script_dir,
      }
    );
  }
}
function getScript(name) {
  if(myController != null && name) {
    myController.sendRequest(
      'fileRead',
      {
        fileName: script_dir + '/' + name,
      }
    );
  }
}
function makeConf(from, to, partyType) {
  if(myController != null && from && to && partyType) {
    myController.sendRequest(
      'taskSendEvent',
      {
        taskRef: taskRef,
        eventName: 'sessMakeCall',
      },
      {
        from: from,
        to: to,
        partyType: partyType,
        confCreate: true,
      }
    );
  }
}
function makeCall(from, to) {
  if(myController != null && from && to) {
    myController.sendRequest(
      'taskSendEvent',
      {
        taskRef: taskRef,
        eventName: 'sessMakeCall',
      },
      {
        from: from,
        to: to,
      }
    );
  }
}
function removeFile(name) {
  if(myController != null && name) {
    myController.sendRequest(
      'fileRemove',
      {
        fileName: name,
      }
    );
  }
}
function getCGCalls(uid, limit) {
  if(myController != null && uid) {
    var param = {};
    param.op = 'getCallsByCardUid';
    param.param = {};
    param.param.uid = uid;
    if(limit) {
      param.param.limit = limit;
    }
    myController.sendRequest(
      'bannerRead',
      {
        type: 'contact',
      },
      JSON.stringify(param)
    );
  }
}
function getSMSTemplates() {
  if(myController != null) {
    myController.sendRequest(
      'taskSendEvent',
      { taskRef: taskRef, eventName: 'sessGetSMSTemplates' }
    );
  }
}
function getSMSTable() {
  if(myController != null) {
    getSettings('smsTemplate', '*');
  }
}
function sendSMS(from, to, text, fileRef) {
  if(myController != null) {
    var params = {};
    params.name = from;
    params.content = text;
    if(fileRef) {
      params.fileRef = fileRef;
    } else {
      params.to = to;
    }
    myController.sendRequest('taskSendEvent', { taskRef: taskRef, eventName: 'sessSendSMS' }, params);
  }
}
function getPrefs() {
  if(myController != null) {
    myController.sendRequest('prefsRead');
  }
}
function checkUpdates(url) {
  if(myController != null) {
    myController.sendRequest('httpCall', { url: url, timeout: 3 });
  }
}
function getStatisticsReport(params) {
  if(myController != null) {
    myController.sendRequest('taskSendEvent', { taskRef: taskRef, eventName: 'sessGetStatistics' }, params);
  }
}
function cancelStatisticsReport() {
  if(myController != null) {
    myController.sendRequest('taskSendEvent', { taskRef: taskRef, eventName: 'sessGetStatistics' }, { operation: 'cancel' });
  }
}
function updateCallNotes(callID, note) {
  if(myController != null) {
    myController.sendRequest('taskSendEvent', { taskRef: taskRef, eventName: 'sessUpdateCallNotes' }, { name: callID, note: note });
  }
}
